#include "main.h"

#ifndef __PMap__
#define __PMap__

struct PMap{
  PMap(interval mu, interval OtermaEnergy) : 
    //vectorField("par:mu,mj,sr;var:x,y,dx,dy;fun:dx,dy,x-mj*(x+mu)*sqrt((x+mu)^2+y^2)^(-3)-mu*(x-mj)*sqrt((x-mj)^2+y^2)^(-3)+2*dy,y*(1-mj*sqrt((x+mu)^2+y^2)^(-3)-mu*sqrt((x-mj)^2+y^2)^(-3))-2*dx;"),
    vectorField("par:mu,mj,sr;var:x,y,dx,dy;fun:dx,dy,x-mj*(x+mu)*((x+mu)^2+y^2)^(-1.5)-mu*(x-mj)*((x-mj)^2+y^2)^(-1.5)+2*dy,y*(1-mj*((x+mu)^2+y^2)^(-1.5)-mu*((x-mj)^2+y^2)^(-1.5))-2*dx;"),
    solver(vectorField,20),
    section(4,1),
    pm(solver,section),
    embeding("par:mu,mj,sr,E;var:x,y,dx,dy;fun:sqrt((x^2+y^2-dx^2)+2*(mj/sqrt((x+mu)^2+y^2)+mu/sqrt((x-mj)^2+y^2)+sr)-E);"),
    Omega("par:mu,mj,sr;var:x,y,dx,dy;fun:0.5*(x^2+y^2)+mj/sqrt((x+mu)^2+y^2)+mu/sqrt((x-mj)^2+y^2)+sr;"),
    Omegax("par:mu,mj,sr;var:x,y,dx,dy;fun:x-mj*(x+mu)/sqrt((x+mu)^2+y^2)^3-mu*(x-mj)/sqrt((x-mj)^2+y^2)^3;"),
    Jacobi("par:mu,mj,sr;var:x,y,dx,dy;fun:x^2+y^2+2*mj/sqrt((x+mu)^2+y^2)+2*mu/sqrt((x-mj)^2+y^2)+2*sr-dx^2-dy^2;"),
    OtermaEnergy(OtermaEnergy)
  {
    setParameters(vectorField,mu);
    setParameters(Omega,mu);
    setParameters(Omegax,mu);
    setParameters(Jacobi,mu);
    setParameters(embeding,mu);
    embeding.setParameter("E",OtermaEnergy);
    //solver.setStep(0.1);
    solver.setAbsoluteTolerance(1e-12);
    solver.setRelativeTolerance(1e-12);
  }

  template<class T>
  void setParameters(T& f, interval mu){
    f.setParameter("mu",mu);
    f.setParameter("mj",1.-mu);
    f.setParameter("sr",mu*(1.-mu)/2.);    
  }
  
  IVector operator()(IVector u, int iterations, double sign){
    capd4::C0HOTripletonSet s(embed(u,sign));
    return project( pm(s,iterations) );
  }
  
  IVector operator()(capd4::IVector x, capd4::IMatrix C, capd4::IVector r, capd4::IMatrix Q, capd4::IVector y, int iterations){
    capd4::C0Rect2Set s(x,C,r);
    interval returnTime;
    return project( pm(s,y,Q,returnTime,iterations) );
  }

  IVector operator()(capd4::IVector x, capd4::IMatrix C, capd4::IVector r, int iterations){
    capd4::C0TripletonSet s(x,C,r);
    return project( pm(s,iterations) );
  }

  IMatrix derivative(IVector u, int iterations, double sign){
    capd4::IVector v = embed(u,sign);
    capd4::C1Rect2Set s(v);
    capd4::IMatrix DQ(4,4);
    capd4::IVector y = pm(s,DQ,iterations);
    DQ = pm.computeDP(y,DQ);

    interval y_prime = sign * sqrt(2*Omega(v)-OtermaEnergy-sqr(v[2]));

    IMatrix DP(2,2);
    DP[0][0] = DQ[0][0] + DQ[0][3] * Omegax(v)/y_prime;
    DP[0][1] = DQ[0][2] - DQ[0][3] * v[2]/y_prime;
    DP[1][0] = DQ[2][0] + DQ[2][3] * Omegax(v)/y_prime;
    DP[1][1] = DQ[2][2] - DQ[2][3] * v[2]/y_prime;
    return DP;
  }

  capd4::IVector embed(IVector u, double sign){
    capd4::IVector v({u[0],interval(0.),u[1],interval(0.)}); 
    v[3] = sign*sqrt(interval(2)*Omega(v)-sqr(v[2])-OtermaEnergy);
    return v;
  }

  IVector project(capd4::IVector u){
    return IVector({u[0],u[2]});
  }

  capd4::IMatrix embed(IVector u, IVector s){
    capd4::IMatrix Q(4,4);
    Q[0][0] = u[0]; Q[2][0] = u[1];
    Q[0][2] = s[0]; Q[2][2] = s[1];
    Q[1][1] = Q[3][3] = 1.;
    return Q;
  }

  capd4::IMap vectorField;
  capd4::IOdeSolver solver;
  capd4::ICoordinateSection section;
  capd4::IPoincareMap pm;
  
  capd4::IFunction embeding;
  capd4::IFunction Omega, Omegax, Jacobi;
  interval OtermaEnergy;
};

#endif
