#include "pcr3bp.h"

// to achieve perfect agreement of the programm with the paper we initiate
// decimal numbers as ratios: example x=0.00357
// will be written as x=interval(357)/interval(1e+5)
// for large integers in the nominator or the denominator it is 
 // necessary to end it by a decimal point, otherwise some compilers 
 // my either protest or convert the number modulo max_integer


// #######################################################################
// definition of sets used for the proof of the existence of L1-Lyapunov orbit
// and its hyperbolicity

void initL1()
{
	Lap10[0] = interval(9208034913207400196.)/interval(1e+19); // L_1 point from the paper
	Lap10[1] = interval(0.);

	IVector S,U;
	S.clear();
	U.clear();

// a new basis, used in the verification of   hyperbolicity 
	U[0] = interval(1.);
	U[1] = interval(25733011.)/interval(1e+7);
	S[0] = interval(-1.);
	S[1] = U[1];


//  quadrangle{H12} -> quadrangle{N[0]}
//  in the paper we have H_1^2 = H12 , N_0 = N[0]

	interval alpha = interval(2.)/interval(1e+7);
	IVector u = U*alpha;
	IVector s = S*alpha;
	H12 = TripleSet(Lap10,s,u,1);

	alpha = interval(3.)/interval(1e+10); // alpha_1 from the paper

// in the paper we have hyperbolic set H_1^1 = H11
	s = S*alpha;
	u = U*alpha;
	interval fuzzy = interval(-1.,1.)*interval(6)/interval(1e+14);
	IVector L = Lap10;
	L[0] += fuzzy;
	H11 = TripleSet(L,s,u,1);

	Lap11[0] = interval(0.9522871276783998976);  // it is not used in rigorous computations
                                    // the estimation of the second intersection
                                    // of the L_1 orbit with x'=0 axis
	Lap11[1] = interval(0.);
}



// #######################################################################
// a definition of sets used for the proof of the existence of L2-Lyapunov orbit
//  and its hyperbolicity

void initL2()
{
	Lap20[0] = interval(1081929486841799903.)/interval(1e+18); // the L_2 point from the paper
	Lap20[1] = interval(0.);

	IVector S,U;
	S.clear();
	U.clear();

	U[0] = interval(1.);
	U[1] = interval(22817915.)/interval(1e+7);
	S[0] = interval(-1.);
	S[1] = U[1];

// hyperbolic set H_2^1  = H21 - in the paper
	interval alpha = interval(4.)/interval(1e+10);
	IVector u = U*alpha;
	IVector s = S*alpha;
	interval fuzzy = interval(-1.,1.)*interval(1.)/interval(1e+13);
	IVector L = Lap20;
	L[0] += fuzzy;
	H21 = TripleSet(L,s,u,-1);

// set H_2^2
	s *= interval(700.);
	u *= interval(30.);
	H22 = TripleSet(Lap20,s,u,-1);

	Lap21[0] = interval(1.046825902265);  // it is not used in rigorous computations
                             // the estimation of the second intersection
                             // of the L_1 orbit with x'=0 axis
	Lap21[1] = interval(0.);
}




// #######################################################################
// a defintion of  N_i's
void initSetsN()
{
	IVector n[9],s[9],u[9];
	for(int i=0;i<9;++i)
	{
		n[i].clear();
		u[i].clear();
		s[i].clear();
	}

// centers of consequtive sets N_i

	n[0][0] = interval(9522928423486199945.)/interval(1e+19);
	n[0][1] = interval(123.)/interval(1e+7);

	n[1][0] = interval(9210057378904251691.)/interval(1e+19);
	n[1][1] = interval(5205932817646883714.)/interval(1e+22);

	n[2][0] = interval(9579163385940664411.)/interval(1e+19);
	n[2][1] = interval(2191497366476494527.)/interval(1e+20);

	n[3][0] = interval(1030069865952822684.)/interval(1e+18);
	n[3][1] = interval(330658676251664686.)/interval(1e+20);

	n[4][0] = interval(9673066820183056081.)/interval(1e+19);
	n[4][1] = interval(3703230165036550462.)/interval(1e+21);

	n[5][0] = interval(1040628850444842879.)/interval(1e+18);
	n[5][1] = interval(2317063455298806405.)/interval(1e+20);

	n[6][0] = interval(1081670357450509546.)/interval(1e+18);
	n[6][1] = interval(5918226490172379421.)/interval(1e+22);

	n[7][0] = interval(1046819673646057103.)/interval(1e+18);
	n[7][1] = interval(2133650650439024551.)/interval(1e+23);

// each of  N[i] has  unstable and stable directions given by u[i], s[i] respectively

	s[0][0] = -interval(4.)/interval(1e+6);
	s[0][1] = interval(145.)/interval(1e+7);
	u[0][0] = -s[0][0];
	u[0][1] = s[0][1];
	u[0] /= 10.;

	s[1][0] = -interval(45.)/interval(1e+8);
	s[1][1] = interval(7.)/interval(6e+6);
	u[1][0] = -s[1][0];
	u[1][1] = s[1][1];
	u[1] /= 10.;

	s[2][0] = -interval(12.)/interval(1e+8);
	s[2][1] = interval(292.)/interval(1e+9);
	u[2][0] = -s[2][0];
	u[2][1] = s[2][1];

	s[3][0] = -interval(105.)/interval(1e+9);
	s[3][1] = interval(292.)/interval(1e+9);
	u[3][0] = -s[3][0];
	u[3][1] = s[3][1];

	s[4][0] = -interval(1.)/interval(1e+7);
	s[4][1] = interval(29.)/interval(1e+8);
	u[4][0] = -s[4][0];
	u[4][1] = s[4][1];
	u[4] /= 2.;

	s[5][0] = -interval(144.)/interval(1e+9);
	s[5][1] = interval(58.)/interval(1e+8);
	u[5][0] = -s[5][0];
	u[5][1] = s[5][1];
	u[5] /= 6.;

	s[6][0] = -interval(1625.)/interval(1e+10);
	s[6][1] = interval(375.)/interval(1e+9);
	u[6][0] = -s[6][0];
	u[6][1] = s[6][1];
	u[6] /= 2.;

	s[7][0] = -interval(83.)/interval(1e+8);
	s[7][1] = interval(29.)/interval(1e+7);
	u[7][0] = -s[7][0];
	u[7][1] = s[7][1];
	u[7] /= 5.;


  double sign = -1;
	for(int i=0;i<=7;++i)
	{
		N[i] = TripleSet(n[i],s[i],u[i],sign);
    sign = -sign;
	}
	
	N[8] = H22; //used only to simplify some loops
}


// #####################################################################
// this function set the h-sets for the proof of the covering relations
// for the homoclinic orbit in interior region

void setInteriorHomoclinic()
{
	IVector H[6],u[6],s[6];
	int i;
	for(i=0;i<6;++i)
		H[i].clear();

	H[0][0] = interval(-6160415155975000064.)/interval(1e+19);
	H[0][1] = interval(0.);

	H[1][0] = interval(8466850372287604736.)/interval(1e+19);
	H[1][1] = interval(1756375376424676608.)/interval(1e+19);

	H[2][0] = interval(9479369578487498752.)/interval(1e+19);
	H[2][1] = interval(1522141990729746432.)/interval(1e+20);

	H[3][0] = interval(9206761120035876864.)/interval(1e+19);
	H[3][1] = interval(32764933375860776.)/interval(1e+20);

	H[4][0] = interval(9522842589493516288.)/interval(1e+19);
	H[4][1] = interval(10481398192083.)/interval(1e+18);

	H[5] = Lap10;

	s[0][0] = interval(1.)/interval(1e+7);
	s[0][1] = -interval(25.)/interval(1e+8);
	u[0][0] = -s[0][0];
	u[0][1] = s[0][1];

	s[1][0] = -interval(1.)/interval(1e+7);
	s[1][1] = -interval(92.)/interval(1e+9);
	u[1][0] = -s[1][0];
	u[1][1] = s[1][1];
	u[1] *= interval(22)/interval(10);

	s[2][0] = -interval(1.)/interval(1e+7);
	s[2][1] = interval(33.)/interval(1e+8);
	u[2][0] = -s[2][0];
	u[2][1] = s[2][1];
	u[2] /= 20.;
	s[2] /= 4.;

	s[3][0] = -interval(1.)/interval(1e+7);
	s[3][1] = interval(26.)/interval(1e+8);
	u[3][0] = -s[3][0];
	u[3][1] = s[3][1];
	u[3] /= 6.;

	s[4][0] = -interval(1.)/interval(1e+7);
	s[4][1] = interval(37.)/interval(1e+8);
	u[4][0] = -s[4][0];
	u[4][1] = s[4][1];
	u[4] /= 6.;

  double sign = -1;
	for(i=0;i<5;++i)
	{
		TS[i] = TripleSet(H[i],s[i],u[i],sign);
    sign = -sign;
	}

	TS[5] = H12;
}

// ######################################################################
// this function initiates the h-sets for the proof of the covering reltions
// for the homoclinic orbit in exterior region

void setExteriorHomoclinic()
{
	IVector H[6],u[6],s[6];
	int i;
	for(i=0;i<6;++i)
		H[i].clear();
	
	H[0][0] = interval(-208509704964865536.)/interval(1e+17);
	H[0][1] = interval(0.);

	H[1][0] = interval(1160261327316386816.)/interval(1e+18);
	H[1][1] = interval(-1812035059427922688.)/interval(1e+19);

	H[2][0] = interval(1059527808809695232.)/interval(1e+18);
	H[2][1] = interval(-3871458787165545984.)/interval(1e+20);

	H[3][0] = interval(1082284499686768768.)/interval(1e+18);
	H[3][1] = interval(-8090412116073312256.)/interval(1e+22);

	H[4][0] = interval(1046834433386131072.)/interval(1e+18);
	H[4][1] = interval(-2957990840481726976.)/interval(1e+23);

	H[5][0] = interval(1081929798158888576.)/interval(1e+18);
	H[5][1] = interval(-7068412578518833152.)/interval(1e+25);


	s[0][0] = interval(1.)/interval(1e+7);
	s[0][1] = -interval(3.)/interval(1e+8);
	u[0][0] = -s[0][0];
	u[0][1] = s[0][1];

	s[1][0] = -interval(1.)/interval(1e+7);
	s[1][1] = -interval(8.)/interval(1e+8);
	u[1][0] = -s[1][0];
	u[1][1] = s[1][1];
	u[1] *= 4.;

	s[2][0] = -interval(3.)/interval(1e+7);
	s[2][1] = interval(81.)/interval(1e+8);
	u[2][0] = -s[2][0];
	u[2][1] = s[2][1];
	u[2] /= 10.;

	s[3][0] = -interval(1.)/interval(1e+7);
	s[3][1] = interval(23.)/interval(1e+8);
	u[3][0] = -s[3][0];
	u[3][1] = s[3][1];
	u[3] /= 4.;

	s[4][0] = -interval(1.)/interval(1e+7);
	s[4][1] = interval(35.)/interval(1e+8);
	u[4][0] = -s[4][0];
	u[4][1] = s[4][1];
	u[4] /= 4.;

	s[5][0] = -interval(1.)/interval(1e+8);
	s[5][1] = interval(22817915.)/interval(1e+15);
	u[5][0] = -s[5][0];
	u[5][1] = s[5][1];
	u[5] /= 2.;

  double sign = 1;
	for(i=0;i<6;++i)
	{
		TS[i] = TripleSet(H[i],s[i],u[i],sign);
    sign = -sign;
	}
}


// #####################################################################
// this function set the h-sets for the proof of the covering reltions
// for the new homoclinic orbit in interior region

void setNewInteriorHomoclinic()
{
	IVector H[6],u[6],s[6];
	int i;
	for(i=0;i<6;++i)
	{
		H[i].clear();
		u[i].clear();
		s[i].clear();
	}

	H[0][0] = 0.5217056203008400006;
	H[1][0] = -0.5822638014577352639; H[1][1] = -0.2793408708392046136;
	H[2][0] = 0.919204446847046941;   H[2][1] = 0.004093829363524479834;
	H[3][0] = 0.9522506335647477061;  H[3][1] = 0.0001333182992547130779;
	H[4][0] = 0.9208022956271231241;  H[4][1] = 2.918364277340028028e-06;
	H[5] = Lap10;

	s[0][0] = interval(1)/interval(1e+7);
	s[0][1] = -interval(2)/interval(1e+7);
	u[0][0] = -s[0][0];
	u[0][1] = s[0][1];

	s[1][0] = -interval(2)/interval(1e+8);
	s[1][1] = interval(4)/interval(1e+7);
	u[1][0] = interval(3)/interval(1e+8);
	u[1][1] = interval(0.);

	s[2][0] = -interval(4)/interval(1e+7);
	s[2][1] = interval(102)/interval(1e+8);
	u[2][0] = -s[2][0];
	u[2][1] = s[2][1];
	u[2] /= 5.;

	s[3][0] = -interval(1)/interval(1e+7);
	s[3][1] = interval(365)/interval(1e+9);
	u[3][0] = -s[3][0];
	u[3][1] = s[3][1];
	u[3] /= 10.;

	s[4][0] = -interval(1)/interval(1e+7);
	s[4][1] = interval(25733011)/interval(1e+14);
	u[4][0] = -s[4][0];
	u[4][1] = s[4][1];
	u[4] /= 2.;

	double sign = 1;
	for(i=0;i<5;++i)
	{
		TS[i] = TripleSet(H[i],s[i],u[i],sign);
    sign = - sign;
	}

	TS[5]=H12;
}

// #####################################################################
// this function set the h-sets for the proof of the covering reltions
// for the 2:3 homoclinic orbit in exterior region

void setNewExteriorHomoclinic()
{
	IVector H[6],u[6],s[6];
	int i;
	for(i=0;i<6;++i)
	{
		H[i].clear();
		u[i].clear();
		s[i].clear();
	}

	H[0][0] = -1.12327231155833984;
	H[1][0] = 1.093337837571255552; H[1][1] = -0.02510094170679043584;
	H[2][0] = 1.047131544421841024; H[2][1] = -0.001056187943513949696;
	H[3][0] = 1.08194053721089792;  H[3][1] = -2.521361165903333888e-05;
	H[4][0] = 1.04682616720451456;  H[4][1] = -9.169345277545603072e-07;
	H[5] = Lap20;

	s[0][0] = interval(1)/interval(1e+8);
	s[0][1] = -interval(40)/interval(1e+8);
	u[0][0] = -s[0][0];
	u[0][1] = s[0][1];

	s[1][0] = -interval(1)/interval(1e+7);
	s[1][1] = interval(21)/interval(1e+8);
	u[1][0] = -s[1][0];
	u[1][1] = s[1][1];
	u[1] /= 10.;

	s[2][0] = -interval(1)/interval(1e+7);
	s[2][1] = interval(35)/interval(1e+8);
	u[2][0] = -s[2][0];
	u[2][1] = s[2][1];
	u[2] /= 10.;

	s[3][0] = -interval(1)/interval(1e+7);
	s[3][1] = interval(23)/interval(1e+8);
	u[3][0] = -s[3][0];
	u[3][1] = s[3][1];
	u[3] /= 10.;

	s[4][0] = -interval(1)/interval(1e+7);
	s[4][1] = interval(35)/interval(1e+8);
	u[4][0] = -s[4][0];
	u[4][1] = s[4][1];
	u[4] /= 4.;

	double sign = 1;
  for(i=0;i<5;++i)
	{
		TS[i] = TripleSet(H[i],s[i],u[i],sign);
    sign = - sign;
	}
	TS[5] = H22;
}

