#include "pcr3bp.h"

// #####################################################################
// the existence of L1 - Lyapunov orbit
// using Darboux theorem and symmetry
// we have to check that \pi_{\dot{x}}P(x_1-\eta_1)<0
// and  \pi_{\dot{x}}P(x_1+\eta_1)>0

void periodicOrbitL1(PMap& pm)
{
	writeReport("Proof of the existence of the Lyapunov orbit L1: ");

// an error for L_1 periodic point
	interval max_error = interval(6.)/interval(1e+14);

  IVector X1({Lap10[0]+max_error,interval(0.)});
  IVector im1 = pm(X1,1,1.);
	if(!(im1[1]>0))
	{
    cout << "im1=" << im1 << endl;
		writeReport("\nError of an estimation of the point L_1");
		exit(1);
	}

  IVector X2({Lap10[0]-max_error,interval(0.)});
  IVector im2 = pm(X2,1,1.);
	if(!(im2[1]<0))
	{
    cout << "im2=" << im2 << endl;
		writeReport("\nError of an estimation of the point L_1");
		exit(1);
	}
	writeReport("done\n\n");
}




// #############################################################################
// the existence of L2-Lyapunov orbit - a function analogous to L1_periodic_orbit()
// using Darboux theorem and symmetry
// we have to check that \pi_{\dot{x}}P(x_2-\eta_2)<0
// and  \pi_{\dot{x}}P(x_2+\eta_2)>0
void periodicOrbitL2(PMap& pm)
{
	writeReport("Proof of the existence of the Lyapunov orbit L2: ");

// an error for periodic point
	interval max_error = interval(1.)/interval(1e+13); // \eta_2
  IVector X1({Lap20[0]+max_error,interval(0.)});
  IVector im1 = pm(X1,1,-1.);
	if(!(im1[1]>0))
	{
		writeReport("\nError of an estimation of the point L_2");
		exit(1);
	}

  IVector X2({Lap20[0]-max_error,interval(0.)});
  IVector im2 = pm(X2,1,-1.);
	if(!(im2[1]<0))
	{
		writeReport("\nError of an estimation of the point L_2");
		exit(1);
	}

	writeReport("done\n\n");
}

