#include "pcr3bp.h"

std::ofstream Report;
const char file[]="report.txt";

IVector Lap10,Lap11,Lap20,Lap21; // L10 and L20 - points close to L_1, L_2 periodic orbits, respectively
TripleSet N[9];                  // triple sets used in the proof the heteroclinic connection
TripleSet H11,H12,H21,H22;       // H11, H21 - hyperbolic sets
TripleSet TS[6];                 // triple sets used in the proof of homoclinic connections
                                 // in the papers are called E_i, F_i, G_i and V_i

// ----------------------------------------------------------------------

void writeReport(std::string s)
{
	std::cout << s << std::flush;
}

/*########################## MAIN ######################################*/

int main(int argc, char *argv[])
{
	std::cout << "Computer assisted proof of the existence of homoclinic and heteroclinic orbits in the PCR3BP\n\n";

  const interval OtermaEnergy=interval(303)/interval(100);
  const interval mu=interval(9537)/interval(1e+7);

  PMap pm(mu,OtermaEnergy);

	try{
// in file ini3bp.cpp
		initL1(); // initialization of h-sets in the neighborhood of L1  
		initL2(); // initialization of h-sets in the neighborhood of L2
		initSetsN(); // initialization of h-sets N_i

// ---------------------------------------------------

// estimate Lyapunov orbits
// in file lyapunov.cpp
		periodicOrbitL1(pm);   // the proof of the existence of L1
		periodicOrbitL2(pm);   // the proof of the existence of L2

// in file cones.cpp
    // check covering relations H11->H11->H12 and cone condition on H11
		checkConeConditionL1(pm);
    // check covering relations H22->H21->H21 and cone condition on H21
		checkConeConditionL2(pm);

// ---------------------------------------------------

// in file connections.cpp
    // the existence of heteroclinic connection between Lyapunov orbits
    // check heteroclinic chain H_1^2 -> N_0 -> ... -> N_7 -> H_2^2
		checkHeteroclinicL1L2(pm);  

    // the existence of two different homoclinic orbits to both L_1 and L_2
    coverInteriorHomoclinic(pm);
		coverExteriorHomoclinic(pm);

		coverNewInteriorHomoclinic(pm);
		coverNewExteriorHomoclinic(pm);
	}
	catch(std::exception &e)
	{
		writeReport(e.what());
		exit(0);
	}
// ----------------------------------------------------

	writeReport("\nProof done.\n");
	return 0;
}
